# -------------------------------------------------------------------------
#
#        P R O D U C T   F I N D I N G
#
# -------------------------------------------------------------------------
import abc
import os
import pathlib
from typing import Iterator

from .products import ExecutionBlock, SpooledProduct


class ProductFinder(abc.ABC):
    """
    Locates products for the delivery to deliver
    """

    @abc.abstractmethod
    def find_products(self) -> Iterator[SpooledProduct]:
        """
        Find products and return an iterator of them

        :return: Iterator of SpooledProducts
        """
        pass


# TODO: Implement after the initial delivery sketch is up and running
# class PiperesultsProductFinder(ProductFinder):
#     def __init__(self, path: pathlib.Path):
#         self.path = path
#
#     def find_products(self) -> Iterator[SpooledProduct]:
#         raise NotImplementedError


class HeuristicProductFinder(ProductFinder):
    def __init__(self, path: pathlib.Path):
        self.path = path

    def find_products(self) -> Iterator[SpooledProduct]:
        """
        Identify the product type and the number of products and returns them as a list

        :return: List of products
        TODO: Search for a `products` directory to identify calibration products; I think that's good enough for now
            to tell EBs from calibrations and once we get more products needing to be delivered we can expand from there
        """
        for subdir in self.path.glob("*"):
            if subdir.is_dir():
                yield ExecutionBlock(subdir)
